use bytemuck;
use std::mem;
use wgpu;

#[repr(C)]
#[derive(Debug, Copy, Clone, Default, bytemuck::Pod, bytemuck::Zeroable)]
// Uniforms require uniform alignment.
// Since the largest value in this array is a [f32; 2],
// all smaller values must be padded.
// also, [f32; 3] are aligned as [f32; 4]
// (since alignments must be powers of two)
pub struct GlobalDataContent {
	/// Camera position x, in game units
	pub camera_position_x: f32,

	/// Camera position y, in game units
	pub camera_position_y: f32,

	/// Camera zoom, in game units
	/// i.e, height if window in game units
	pub camera_zoom: f32,

	/// Minimum camera zoom
	pub camera_zoom_min: f32,

	/// Maximum camera zoom
	pub camera_zoom_max: f32,

	/// Window width, in physical pixels
	pub window_size_w: f32,

	/// Window height, in physical pixels
	pub window_size_h: f32,

	/// Physical to logical pixel conversion factor
	pub window_scale: f32,

	/// Aspect ratio of window
	pub window_aspect: f32,

	/// Index of starfield sprite
	pub starfield_sprite: u32,

	/// Size of starfield tile squares
	pub starfield_tile_size: f32,

	/// Minimum starfield star size
	pub starfield_size_min: f32,

	/// Maximum starfield star size
	pub starfield_size_max: f32,
}

impl GlobalDataContent {
	pub const SIZE: u64 = mem::size_of::<Self>() as wgpu::BufferAddress;
}
