use log::error;
use rhai::{FnNamespace, FuncRegistration, ImmutableString, Module};
use std::{cell::RefCell, rc::Rc};

use super::super::{Color, Rect};
use crate::ui::{elements::RadialBar, UiElement, UiState};

pub fn build_radialbar_module(state_src: Rc<RefCell<UiState>>) -> Module {
	let mut module = Module::new();
	module.set_id("GalacticaRadialbarModule");

	let state = state_src.clone();
	let _ = FuncRegistration::new("add")
		.with_namespace(FnNamespace::Internal)
		.set_into_module(
			&mut module,
			// TODO: fix ugly spaces
			move |name: ImmutableString, stroke: f32, color: Color, rect: Rect| {
				let mut ui_state = state.borrow_mut();

				if ui_state.elements.contains_key(&name) {
					error!("tried to make a radialbar using an existing name `{name}`");
					return;
				}

				ui_state.names.push(name.clone());
				ui_state.elements.insert(
					name.clone(),
					UiElement::RadialBar(RadialBar::new(name.clone(), stroke, color, rect, 1.0)),
				);
			},
		);

	let state = state_src.clone();
	let _ = FuncRegistration::new("set_val")
		.with_namespace(FnNamespace::Internal)
		.set_into_module(&mut module, move |name: ImmutableString, val: f32| {
			let mut ui_state = state.borrow_mut();
			match ui_state.get_mut_by_name(&name) {
				Some(UiElement::RadialBar(x)) => x.set_val(val),
				_ => {
					error!("called `radialbar_set_val` on an invalid name `{name}`")
				}
			}
		});

	return module;
}
