mod event;
mod functions;
mod helpers;
mod state;

pub use event::*;
use glyphon::FontSystem;
pub use helpers::{anchor::*, color::*, rect::*, vector::*};
use log::debug;
pub use state::*;

use super::UiState;
use galactica_content::Content;
use rhai::{exported_module, Dynamic, Engine};
use std::{cell::RefCell, rc::Rc, sync::Arc};

pub fn register_into_engine(
	engine: &mut Engine,
	ct_src: Arc<Content>,
	font_src: Rc<RefCell<FontSystem>>,
	state_src: Rc<RefCell<UiState>>,
) {
	engine
		// Helpers
		.build_type::<Rect>()
		.build_type::<Color>()
		.build_type::<UiVector>()
		// State
		.build_type::<State>()
		.build_type::<ShipState>()
		.build_type::<SystemObjectState>()
		// Events
		.build_type::<MouseClickEvent>()
		.build_type::<MouseHoverEvent>()
		.build_type::<PlayerShipStateEvent>()
		// Bigger modules
		.register_type_with_name::<Anchor>("Anchor")
		.register_static_module("Anchor", exported_module!(anchor_mod).into());

	// Extra functions
	engine.register_fn("print", move |d: Dynamic| {
		debug!("{:?}", d);
	});
	engine.register_fn("clamp", move |x: f32, l: f32, h: f32| x.clamp(l, h));

	// Modules
	engine.register_static_module(
		"sprite",
		functions::build_sprite_module(ct_src.clone(), state_src.clone()).into(),
	);
	engine.register_static_module(
		"textbox",
		functions::build_textbox_module(font_src.clone(), state_src.clone()).into(),
	);
	engine.register_static_module(
		"radialbar",
		functions::build_radialbar_module(state_src.clone()).into(),
	);
	engine.register_static_module("ui", functions::build_ui_module(state_src.clone()).into());
	engine.register_static_module(
		"conf",
		functions::build_conf_module(state_src.clone()).into(),
	);
}
