use glyphon::{Attrs, Buffer, Color, Family, FontSystem, Metrics, Shaping, TextArea, TextBounds};
use winit::window::Window;

use crate::{RenderInput, RenderState};

pub(crate) struct FpsIndicator {
	buffer: Buffer,
	update_counter: u32,
}

impl FpsIndicator {
	pub fn new(state: &mut RenderState) -> Self {
		let mut buffer = Buffer::new(
			&mut state.text_font_system.borrow_mut(),
			Metrics::new(7.0, 8.0),
		);
		buffer.set_size(
			&mut state.text_font_system.borrow_mut(),
			state.window_size.width as f32,
			state.window_size.height as f32,
		);

		Self {
			buffer,
			update_counter: 0,
		}
	}
}

impl FpsIndicator {
	pub fn step(&mut self, input: &RenderInput, font: &mut FontSystem) {
		if self.update_counter > 0 {
			self.update_counter -= 1;
			return;
		}
		self.update_counter = 100;

		self.buffer.set_text(
			font,
			&input.timing.get_string(),
			Attrs::new().family(Family::Monospace),
			Shaping::Basic,
		);
		self.buffer.shape_until_scroll(font);
	}
}

impl<'a, 'b: 'a> FpsIndicator {
	pub fn get_textarea(&'b self, input: &RenderInput, _window: &Window) -> TextArea<'a> {
		TextArea {
			buffer: &self.buffer,
			left: 10.0,
			top: 400.0,
			scale: input.ct.config.ui_scale,
			bounds: TextBounds {
				left: 10,
				top: 400,
				right: 300,
				bottom: 800,
			},
			default_color: Color::rgb(255, 255, 255),
		}
	}
}
