use super::{ShipControllerStruct, ShipControls};
use crate::phys::{PhysImage, PhysSimShipHandle, PhysStepResources};

/// The Null controller is assigned to objects that are static or not controlled by the computer.
/// Most notably, the player's ship has a Null controller.
#[derive(Debug, Clone)]
pub struct NullShipController {}

impl NullShipController {
	/// Create a new ship controller
	pub fn new() -> Self {
		Self {}
	}
}

impl ShipControllerStruct for NullShipController {
	fn update_controls(
		&mut self,
		_res: &PhysStepResources,
		_img: &PhysImage,
		_this_ship: PhysSimShipHandle,
	) -> Option<ShipControls> {
		None
	}
}
